#!/bin/bash
clear
script_path="MoE-MERC"
cd "$script_path"
num_gpus=1
source "$script_path"/scripts/configs/config.sh ${num_gpus}
# export CUDA_VISIBLE_DEVICES="1,2,4"

project="MoE-MERC"
time=$(date "+%Y%m%d-%H%M%S")
model_name="dq_former"
dataset_name=$1
class_type="emotion"
label_type=$2
text_model_name='bert_base_uncased'
audio_model_name='wavlm_base'
fusion_layers=4
num_layers=4
conetxt_window_size=5
bottleneck_len=16
num_heads=8
batch_size=4
gradient_accumulation_steps=1
split_type="dialog"
seed=$3
tau=0.1

if [[ $text_model_name == *"base"* ]]; then
  intermediate_dim=768
else
  intermediate_dim=1024
fi

if [[ $text_model_name == *"llama"* ]]; then
  intermediate_dim=4096
else
  intermediate_dim=768
fi

text_variable="${text_model_name}"
text_model=$(eval echo "\$$text_variable")
echo "Path for $text_variable: $text_model"
audio_variable="${audio_model_name}"
audio_model=$(eval echo "\$$audio_variable")
echo "Path for $audio_variable: $audio_model"

group_name=${dataset_name}-${model_name}
run_name=${text_model_name}-${audio_model_name}-"l"-${num_layers}-"c"-${conetxt_window_size}
echo "Group name: $group_name"
echo "Run name: $run_name"


python train.py \
    --wandb_project ${project} \
    --wandb_run_name ${run_name} \
    --wandb_group_name ${group_name} \
    --model_name ${model_name} \
    --dataset_name ${dataset_name} \
    --dataset_root_dir ${DATASET_ROOT_DIR} \
    --label_type ${label_type} \
    --class_type ${class_type} \
    --text_model ${text_model} \
    --audio_model ${audio_model} \
    --audio_processor ${wav2vec2_base} \
    --num_layers ${num_layers} \
    --num_heads ${num_heads} \
    --fusion_layers ${fusion_layers} \
    --context_window_size ${conetxt_window_size} \
    --split_type ${split_type} \
    --bottleneck_len ${bottleneck_len} \
    --intermediate_dim ${intermediate_dim} \
    --batch_size ${batch_size} \
    --gradient_accumulation_steps ${gradient_accumulation_steps} \
    --num_epochs 20 \
    --learning_rate 1e-5 \
    --warmup_proportion 0.01 \
    --max_seq_len 512 \
    --time_durations 10.0 \
    --tau ${tau} \
    --early_stopping_patience 7 \
    --do_train True \
    --save_total_limit 2 \
    --seed ${seed} \
    --enable_modality_embedding True \
